from django.db import models
from django.http import Http404
from django.core.validators import RegexValidator,EmailValidator

# Create your models here.

class Passenger(models.Model):
    P_ID = models.AutoField(primary_key=True, editable=False, unique=True)
    First_Name = models.CharField(max_length=50)
    Middle_Initial = models.CharField(max_length=50, blank=True)
    Last_Name = models.CharField(max_length = 50)
    Birthdate = models.DateField(null=True, blank=True)
    Age = models.CharField(max_length=3)
    male = 'Male'
    female = 'Female'
    na = 'Prefer not to say'
    gender_choices = [
        (male, 'Male'),
        (female, 'Female'),
        (na, 'Prefer not to say'),
    ]
    Gender = models.CharField(
        max_length=20,
        choices=gender_choices,
        default=na,
        )

    Phone_num = models.CharField(max_length = 11, validators=[RegexValidator(r'^[0-9]*$',message = "Please input numbers only")])
    Email_add = models.EmailField(max_length = 254, validators=[EmailValidator()])
    
    def __str__(self):
        return self.First_Name + ' ' + self.Last_Name


class Booking(models.Model):
    Booking_ID = models.AutoField(primary_key=True, editable=False, unique=True)
    Booking_Date = models.DateField(null=True, blank=True)
    Total_Cost = models.IntegerField(blank=True, default=0,null=False)
    Passenger = models.ForeignKey(Passenger, on_delete=models.CASCADE)

    def __str__(self):
        return str(self.Booking_ID)
   
        
class Additional_Item(models.Model):
    Description = models.CharField(max_length=50, primary_key=True, editable=True, unique=True)
    Item_Cost = models.IntegerField(default=0)

    def __str__(self):
        return self.Description 

class Booking_AddItem(models.Model):
    class Meta:
        unique_together = ('Booking_ID', 'AddItem_Desc')

    Booking_ID = models.ForeignKey(Booking, on_delete=models.CASCADE)
    AddItem_Desc = models.ForeignKey(Additional_Item, on_delete=models.CASCADE)
    Quantity = models.IntegerField(default=0)

    @property
    def subtotal(self):
        add_item = Additional_Item.objects.get(Description = self.AddItem_Desc)
        itemcost = add_item.Item_Cost
        subtotal = itemcost*self.Quantity
        return subtotal

    

class City(models.Model):
    Airport_Code = models.AutoField(primary_key=True, editable=False, unique=True)
    Airport_name = models.CharField(max_length=50)
    City_Name = models.CharField(max_length=50)
    Country = models.CharField(max_length=50)

    def __str__(self):
        return self.Airport_name + ' (' + self.Country + ')'

class Crew_Member(models.Model):
    Employee_ID = models.AutoField(primary_key=True, editable=False, unique=True)
    First_Name = models.CharField(max_length=50)
    Last_Name = models.CharField(max_length=50)

    pilot = 'Pilot'
    flight_attendant = 'Flight Attendant'
    role_choices = [
        (pilot, 'Pilot'),
        (flight_attendant, 'Flight Attendant')
    ]

    Role = models.CharField(max_length=50, choices=role_choices)

    def __str__(self):
        return self.First_Name + ' ' + self.Last_Name + ' (' + self.Role + ')'

class Pilot(models.Model):
    Employee_ID = models.OneToOneField(Crew_Member, on_delete=models.CASCADE, primary_key=True)
    cadet_pilot = 'Cadet Pilot'
    second_officer = 'Second Officer'
    first_officer = 'First Officer'
    senior_firstofficer = 'Senior First Officer'
    captain = 'Captain'
    rank_choices = [
        (cadet_pilot, 'Cadet Pilot'),
        (second_officer, 'Second Officer'),
        (first_officer, 'First Officer'),
        (senior_firstofficer, 'Senior First Officer'),
        (captain, 'Captain')
        ]
    Rank = models.CharField(max_length=50, choices=rank_choices)

    def __str__(self):
        return self.Employee_ID.First_Name + ' ' + self.Employee_ID.Last_Name + ' (' + self.Rank + ')'

class Flight_Attendant(models.Model):
    Employee_ID = models.OneToOneField(Crew_Member, on_delete=models.CASCADE, primary_key=True)
    economy = 'Economy'
    premium_economy = 'Premium Economy'
    business = 'Business'
    first_class = 'First Class'
    class_choices = [
        (economy, 'Economy'),
        (premium_economy, 'Premium Economy'),
        (business, 'Business'),
        (first_class, 'First Class')
    ]
    Assigned_Class = models.CharField(max_length=50, choices=class_choices)
    
    def __str__(self):
        return self.Employee_ID.First_Name + ' ' + self.Employee_ID.Last_Name + ' (' + self.Assigned_Class + ')'

class Flight(models.Model):
    Flight_ID = models.AutoField(primary_key=True, editable=False, unique=True)
    Flight_Code = models.CharField(max_length=6)
    Origin_Airport = models.ForeignKey(City, on_delete=models.CASCADE, related_name='origin')
    Destination_Airport = models.ForeignKey(City, on_delete=models.CASCADE, related_name='destination')

    def __str__(self):
        return ' (' + self.Flight_Code + ' ' + self.Origin_Airport.City_Name + ' to ' + self.Destination_Airport.City_Name

class Schedule(models.Model):
    Sched_Code = models.AutoField(primary_key=True, editable=False, unique=True)
    Flight_Date = models.DateField(null=True, blank=True)
    Departure_Time = models.TimeField(null=True, blank=True)
    Arrival_Time = models.TimeField(null=True, blank=True)
    Duration = models.CharField(max_length=50)
    Flight_Cost = models.CharField(max_length=50)
    Flight_Code = models.ForeignKey(Flight, on_delete=models.CASCADE)

class Sched_Crew(models.Model):
    class Meta:
        unique_together = ('Sched_Code', 'Employee_ID')

    Sched_Code = models.ForeignKey(Schedule, on_delete=models.CASCADE)
    Employee_ID = models.ForeignKey(Crew_Member, on_delete=models.CASCADE)

class Booking_Sched(models.Model):
    class Meta:
        unique_together = ('Booking_ID', 'Sched_Code')

    Booking_ID = models.ForeignKey(Booking, on_delete=models.CASCADE)
    Sched_Code = models.ForeignKey(Schedule, on_delete=models.CASCADE)
